"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from .httpclient import AsyncHttpClient, HttpClient
from .sdkconfiguration import SDKConfiguration
from .utils.logger import Logger, get_default_logger
from .utils.retries import RetryConfig
from codat_sync_for_payables import utils
from codat_sync_for_payables._hooks import SDKHooks
from codat_sync_for_payables.bank_accounts import BankAccounts
from codat_sync_for_payables.bill_payments import BillPayments
from codat_sync_for_payables.bills import Bills
from codat_sync_for_payables.companies import Companies
from codat_sync_for_payables.company_information import CompanyInformation
from codat_sync_for_payables.connections import Connections
from codat_sync_for_payables.models import shared
from codat_sync_for_payables.suppliers import Suppliers
from codat_sync_for_payables.types import OptionalNullable, UNSET
import httpx
from typing import Callable, Dict, Optional, Union


class CodatSyncPayables(BaseSDK):
    r"""Bill pay kit: The API reference for the Bill Pay kit.

    The bill pay kit is an API and a set of supporting tools designed to integrate a bill pay flow into your app as quickly as possible. It's ideal for facilitating essential bill payment processes within your SMB's accounting software.

    [Explore product](https://docs.codat.io/payables/bill-pay-kit) | [See OpenAPI spec](https://github.com/codatio/oas)

    ---
    ## Supported Integrations

    | Integration                   | Supported |
    |-------------------------------|-----------|
    | FreeAgent                     | Yes       |
    | QuickBooks Online             | Yes       |
    | Oracle NetSuite               | Yes       |
    | Xero                          | Yes       |

    ---
    <!-- Start Codat Tags Table -->
    ## Endpoints

    | Endpoints | Description |
    | :- |:- |
    | Companies | Create and manage your SMB users' companies. |
    | Connections | Create new and manage existing data connections for a company. |
    | Company information | View company profile from the source platform. |
    | Bills | Get, create, and update Bills. |
    | Bill payments | Get, create, and update Bill payments. |
    | Suppliers | Get, create, and update Suppliers. |
    | Bank accounts | Create a bank account for a given company's connection. |
    <!-- End Codat Tags Table -->
    """

    companies: Companies
    r"""Create and manage your SMB users' companies."""
    connections: Connections
    r"""Create new and manage existing data connections for a company."""
    company_information: CompanyInformation
    r"""View company profile from the source platform."""
    bills: Bills
    r"""Get, create, and update Bills."""
    bill_payments: BillPayments
    r"""Get, create, and update Bill payments."""
    suppliers: Suppliers
    r"""Get, create, and update Suppliers."""
    bank_accounts: BankAccounts
    r"""Create a bank account for a given company's connection."""

    def __init__(
        self,
        security: Union[shared.Security, Callable[[], shared.Security]],
        server_idx: Optional[int] = None,
        server_url: Optional[str] = None,
        url_params: Optional[Dict[str, str]] = None,
        client: Optional[HttpClient] = None,
        async_client: Optional[AsyncHttpClient] = None,
        retry_config: OptionalNullable[RetryConfig] = UNSET,
        timeout_ms: Optional[int] = None,
        debug_logger: Optional[Logger] = None,
    ) -> None:
        r"""Instantiates the SDK configuring it with the provided parameters.

        :param security: The security details required for authentication
        :param server_idx: The index of the server to use for all methods
        :param server_url: The server URL to use for all methods
        :param url_params: Parameters to optionally template the server URL with
        :param client: The HTTP client to use for all synchronous methods
        :param async_client: The Async HTTP client to use for all asynchronous methods
        :param retry_config: The retry configuration to use for all supported methods
        :param timeout_ms: Optional request timeout applied to each operation in milliseconds
        """
        if client is None:
            client = httpx.Client()

        assert issubclass(
            type(client), HttpClient
        ), "The provided client must implement the HttpClient protocol."

        if async_client is None:
            async_client = httpx.AsyncClient()

        if debug_logger is None:
            debug_logger = get_default_logger()

        assert issubclass(
            type(async_client), AsyncHttpClient
        ), "The provided async_client must implement the AsyncHttpClient protocol."

        if server_url is not None:
            if url_params is not None:
                server_url = utils.template_url(server_url, url_params)

        BaseSDK.__init__(
            self,
            SDKConfiguration(
                client=client,
                async_client=async_client,
                security=security,
                server_url=server_url,
                server_idx=server_idx,
                retry_config=retry_config,
                timeout_ms=timeout_ms,
                debug_logger=debug_logger,
            ),
        )

        hooks = SDKHooks()

        current_server_url, *_ = self.sdk_configuration.get_server_details()
        server_url, self.sdk_configuration.client = hooks.sdk_init(
            current_server_url, self.sdk_configuration.client
        )
        if current_server_url != server_url:
            self.sdk_configuration.server_url = server_url

        # pylint: disable=protected-access
        self.sdk_configuration.__dict__["_hooks"] = hooks

        self._init_sdks()

    def _init_sdks(self):
        self.companies = Companies(self.sdk_configuration)
        self.connections = Connections(self.sdk_configuration)
        self.company_information = CompanyInformation(self.sdk_configuration)
        self.bills = Bills(self.sdk_configuration)
        self.bill_payments = BillPayments(self.sdk_configuration)
        self.suppliers = Suppliers(self.sdk_configuration)
        self.bank_accounts = BankAccounts(self.sdk_configuration)

    def __enter__(self):
        return self

    async def __aenter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        if self.sdk_configuration.client is not None:
            self.sdk_configuration.client.close()

    async def __aexit__(self, exc_type, exc_val, exc_tb):
        if self.sdk_configuration.async_client is not None:
            await self.sdk_configuration.async_client.aclose()
